//+------------------------------------------------------------------+
//|                                                 Wilder's DMI.mq4 |
//|                                                  coded by mladen |
//|                                                                  |
//| Directional movement index was developed by Welles Wilder        |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property  indicator_buffers   3
#property indicator_color1     LimeGreen
#property indicator_color2     Red
#property indicator_color3     Orange
#property indicator_width3     2
#property indicator_minimum    0

//
//
//
//
//
enum enTimeFrames
{
   tf_cu  = PERIOD_CURRENT, // Current time frame
   tf_m1  = PERIOD_M1,      // 1 minute
   tf_m5  = PERIOD_M5,      // 5 minutes
   tf_m15 = PERIOD_M15,     // 15 minutes
   tf_m30 = PERIOD_M30,     // 30 minutes
   tf_h1  = PERIOD_H1,      // 1 hour
   tf_h4  = PERIOD_H4,      // 4 hours
   tf_d1  = PERIOD_D1,      // Daily
   tf_w1  = PERIOD_W1,      // Weekly
   tf_mn1 = PERIOD_MN1,     // Monthly
   tf_n1  = -1,             // First higher time frame
   tf_n2  = -2,             // Second higher time frame
   tf_n3  = -3              // Third higher time frame
};
//
//

enum enPrices
{
   pr_close,      // Close
   pr_open,       // Open
   pr_high,       // High
   pr_low,        // Low
   pr_median,     // Median
   pr_typical,    // Typical
   pr_weighted    // Weighted
};
enum enAdxType
{
   adx_wil, // Wilders DMI (ADX)
   adx_bui  // Built in ADX
};   

extern enTimeFrames    TimeFrame          = tf_cu;   // Time frame
extern int             AdxPeriod          = 14;
extern enAdxType       AdxType            = adx_wil;
extern enPrices        PriceForBuiltInADX = pr_close;
extern bool            alertsOn           = true;
extern bool            alertsOnCurrent    = false;
extern bool            alertsMessage      = true;
extern bool            alertsSound        = false;
extern bool            alertsEmail        = false;
extern bool            alertsNotify       = false;
extern double          Level              = 20.0;
extern bool            Show_Dmi_Only      = false; 

//
//
//
//
//

double DIp[];
double DIm[];
double ADX[];
double trend[];

string indicatorFileName;
bool   returnBars;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//

int init()
{
   IndicatorBuffers(4);
   SetIndexBuffer(0,DIp);    SetIndexLabel(0,"DI+");
   SetIndexBuffer(1,DIm);    SetIndexLabel(1,"DI-");
   SetIndexBuffer(2,ADX);    SetIndexLabel(2,"ADX");
   SetIndexBuffer(3,trend);
         for (int i=0;i<3;i++) SetIndexEmptyValue(i,0.00);
         SetLevelValue(0,Level);
         if (Show_Dmi_Only)
               SetIndexStyle(2,DRAW_NONE); 
         else  SetIndexStyle(2,DRAW_LINE);
   
         //
         //
         //
         //
         //
     
         indicatorFileName = WindowExpertName();
         returnBars        = TimeFrame==-99;
         TimeFrame         = (enTimeFrames)timeFrameValue(TimeFrame);
        
   string name = " Wilder\'s "; if (AdxType==adx_bui) name = "";
   IndicatorShortName(timeFrameToString(TimeFrame)+name+" ADX ("+AdxPeriod+")");
return(0);
}

int deinit() { return(0);}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double work[][3];
#define _DIp 0
#define _DIm 1
#define _TR  2

int start()
{
   int i,r,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-2);
           if (returnBars) { DIp[0] = limit+1; return(0); }
         
   //
   //
   //
   //
   //
   
   if (TimeFrame==Period())
   {
      if (ArrayRange(work,0)!=Bars) ArrayResize(work,Bars);
      double sf = (AdxPeriod-1.0)/AdxPeriod;
      for (i=limit, r=Bars-i-1;i>=0;i--,r++)
      {
         if (AdxType==adx_bui)
         {
            ADX[i] = iADX(NULL,0,AdxPeriod,(int)PriceForBuiltInADX,MODE_MAIN,i);
            DIp[i] = iADX(NULL,0,AdxPeriod,(int)PriceForBuiltInADX,MODE_PLUSDI,i);
            DIm[i] = iADX(NULL,0,AdxPeriod,(int)PriceForBuiltInADX,MODE_MINUSDI,i);
            continue;
         }
         double currTR  = MathMax(High[i],Close[i+1])-MathMin(Low[i],Close[i+1]);
         double DeltaHi = High[i]  - High[i+1];
	      double DeltaLo = Low[i+1] - Low[i];
         double plusDM  = 0.00;
         double minusDM = 0.00;
         
            if ((DeltaHi > DeltaLo) && (DeltaHi > 0)) plusDM  = DeltaHi;
            if ((DeltaLo > DeltaHi) && (DeltaLo > 0)) minusDM = DeltaLo;      
         
         //
         //
         //
         //
         //
         
            work[r][_DIp] = sf*work[r-1][_DIp] + plusDM;
            work[r][_DIm] = sf*work[r-1][_DIm] + minusDM;
            work[r][_TR]  = sf*work[r-1][_TR]  + currTR;

         //
         //
         //
         //
         //
                  
            DIp[i] = 0.00;                   
            DIm[i] = 0.00;                   
            if (work[r][_TR] > 0)             
            {              
               DIp[i] = 100.00 * work[r][_DIp]/work[r][_TR];
               DIm[i] = 100.00 * work[r][_DIm]/work[r][_TR];
            }            
              double DX;
              if((DIp[i] + DIm[i])>0) 
                   DX  = 100*MathAbs(DIp[i] - DIm[i])/(DIp[i] + DIm[i]); 
              else DX  = 0.00;
              ADX[i]   = sf*ADX[i+1] + DX/AdxPeriod; 
              trend[i] = trend[i+1];
              if (DIp[i]>DIm[i]) trend[i] = 1;
              if (DIp[i]<DIm[i]) trend[i] =-1;
         }
         
         //
         //
         //
         //
         //
     
         if (alertsOn)
         {
           if (alertsOnCurrent)
                int whichBar = 0;
           else     whichBar = 1;
           if (trend[whichBar] != trend[whichBar+1])
           if (trend[whichBar] == 1)
                 doAlert("diPlus crossing diMinus up");
           else  doAlert("diPlus crossing diMinus down");       
   }
         return(0);
   } 
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period()));
   for (i=limit;i>=0; i--)
   {
      int y = iBarShift(NULL,TimeFrame,Time[i]);
         DIp[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,AdxPeriod,AdxType,PriceForBuiltInADX,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsNotify,0,y);
         DIm[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,AdxPeriod,AdxType,PriceForBuiltInADX,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsNotify,1,y);
         ADX[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,AdxPeriod,AdxType,PriceForBuiltInADX,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsNotify,2,y);       
   }
return(0);
} 

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}
int timeFrameValue(int _tf)
{
   int add  = (_tf>=0) ? 0 : MathAbs(_tf);
   if (add != 0) _tf = _Period;
   int size = ArraySize(iTfTable); 
      int i =0; for (;i<size; i++) if (iTfTable[i]==_tf) break;
                                   if (i==size) return(_Period);
                                                return(iTfTable[(int)MathMin(i+add,size-1)]);
}
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];
          
          //
          //
          //
          //
          //

          message =  StringConcatenate(timeFrameToString(_Period)+" "+Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Wilders Adx ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Wilders Adx"),message);
             if (alertsNotify)  SendNotification(message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
}